/*********************************************************************

CSE 142
Homework 4  Part 2

Title:      Traffic!
Date:       5-12-99

Author:     Ryan Card

E-mail:     mrcard@u.washington.edu
Section:    BA

*********************************************************************/



/* INCLUDES */

#include "gp142.h"
#include <stdio.h>
#include <math.h>
#include <time.h>



/* DEFINES */

/* Logical constants. */
#define TRUE                 1
#define FALSE                0

/* In GP142, thickness of 0 means fill. */
#define FILL                 0

/* Possible mouse clicks. */
#define CLICK_NOWHERE        0
#define CLICK_GO             1
#define CLICK_STOP           2
#define CLICK_REVERSE        3
#define CLICK_NUMCAR_UP      4
#define CLICK_NUMCAR_DOWN    5
#define CLICK_SPEED_UP       6
#define CLICK_SPEED_DOWN     7
#define CLICK_PALETTE        8

/* Traffic movement constants. */
#define STOP                 0
#define FORWARD              1
#define REVERSE              2

/* Constants about buttons. */
#define BUTTON_HEIGHT        30
#define BUTTON_WIDTH         80
#define BUTTON_SPACING       5
#define BUTTON_LEFT          BUTTON_SPACING-GP142_XMAX
#define BUTTON_UP            GP142_YMAX-BUTTON_SPACING
#define BUTTON_TEXT_OFFSET   10-GP142_XMAX
#define BUTTON_TEXT_SIZE     13
#define NUM_BUTTONS          3

/* Constants about control buttons. */
#define CONTROL_HEIGHT       BUTTON_HEIGHT
#define CONTROL_WIDTH        BUTTON_WIDTH
#define CONTROL_LEFT         BUTTON_LEFT
#define CONTROL_UP           BUTTON_UP-(BUTTON_HEIGHT+BUTTON_SPACING)*NUM_BUTTONS
#define CONTROL_BUTTON_WIDTH 20
#define CONTROL_BUTTON_LEFT  CONTROL_LEFT+CONTROL_WIDTH+BUTTON_SPACING
#define CONTROL_BUTTON_UP    CONTROL_UP
#define CONTROL_TEXT_OFFSET  BUTTON_TEXT_OFFSET
#define CONTROL_TEXT_SIZE    BUTTON_TEXT_SIZE
#define NUM_CONTROL          2

/* Constants about time display. */
#define TIME_HEIGHT          BUTTON_HEIGHT
#define TIME_WIDTH           BUTTON_WIDTH
#define TIME_LEFT            BUTTON_LEFT
#define TIME_UP              CONTROL_UP-(CONTROL_HEIGHT+BUTTON_SPACING)*NUM_CONTROL
#define TIME_TEXT_OFFSET     18-GP142_XMAX
#define TIME_TEXT_SIZE       20

/* Constants about palette display. */
#define PALETTE_COLOR_WIDTH  20
#define PALETTE_HEIGHT       50
#define PALETTE_WIDTH        PALETTE_COLOR_WIDTH*MAX_COLORS
#define PALETTE_LEFT         BUTTON_LEFT
#define PALETTE_UP           -GP142_YMAX+PALETTE_HEIGHT+BUTTON_SPACING
#define PALETTE_TEXT_OFFSET  BUTTON_TEXT_OFFSET
#define PALETTE_TEXT_SIZE    20

/* Background related constants. */
#define HORIZON              0
#define NUM_STARS            10
#define NUM_RAYS             5
#define SATELLITE_RADIUS     15

/* Road coordinates.  The combination of different x's and y's account
   for possible intersection locations. */
#define ROAD_WIDTH           20
#define ROAD_TOP_Y           -3*ROAD_WIDTH
#define ROAD_BOTTOM_Y        -8*ROAD_WIDTH

/* Extreme values for components of city. */
#define MAX_VEHICLES         20
#define MAX_SPEED_LIMIT      100
#define MIN_SPEED_LIMIT      15

/* Vehicle types. */
#define CAR                  1
#define SPORTS               2
#define TRUCK                3

/* Vehicle abundance probability. */
#define CHANCE_CAR           50
#define CHANCE_SPORTS        30
#define CHANCE_TRUCK         20

/* Vehicle dimentions. */
#define CAR_LENGTH           40
#define CAR_HEIGHT           25
#define SPORTS_LENGTH        40
#define SPORTS_HEIGHT        25
#define TRUCK_LENGTH         50
#define TRUCK_HEIGHT         30

/* Vehicle states. */
#define STATE_NORMAL         1
#define STATE_HONKING        2

/* City initializations. */
#define INIT_NUMCARS_ALLOWED 10
#define INIT_NUMCARS         0
#define INIT_TRAFFIC_MOVING  TRUE
#define INIT_SPEED_LIMIT     25
#define INIT_PALETTE         RED
#define INIT_HOURS           12
#define INIT_MINUTES         0

/* Time constants. */
#define HOUR_MORNING         6
#define HOUR_DUSK            18
#define MINUTE_INCREMENT     1


/* DEFINITIONS */

typedef struct
{
	int numcars_allowed;
	int numcars;
	int traffic_moving;
	int speed_limit;
	int palette;
	int hour, minute;
} CITY;


typedef struct
{
	int x;
	int y;
	char direction;
	int type;
	int color;
	int state;
	int state_count;
} VEHICLE;



/* FUNCTION PROTOTYPES */

/* Draw the background. */
void draw_landscape(CITY ryanville);

/* Draw few randomly positioned stars. */
void sprinkle_star(void);

/* Draw Sun/Moon.  Position calculated by time. */
void draw_satellite(CITY ryanville, int sky_color);

/* Draw dashed yellow road center line horizontally. */
void draw_center_line(int y);

/* Draw all of the buttons. */
void draw_buttons(CITY ryanville);

/* Return kind of mouse click. */
int classify_mouse_click(int mouse_x, int mouse_y);

/* Process button click at location x, y. */
void handle_button(int mouse_x, int mouse_y,
				   CITY *ryanville, VEHICLE *vehicles);

/* Check to see if the coordinates x, y overlaps with another car. */
/* If it does overlaps, return the index of the car it overlaps */
/* with, but if it doesn't overlap, -1 is returned. */
int check_car(VEHICLE *vehicles, CITY ryanville,
			  int mouse_x, int mouse_y);

/* Returns the color clicked in the palette. */
int color_clicked(int mouse_x, int mouse_y);

/* Update the time. */
void changetime(CITY *ryanville);

/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick. */
void move_traffic(VEHICLE *vehicles, CITY *ryanville);

/* Randomly generate a car. */
void create_car(VEHICLE *vehicles, CITY *ryanville);

/* Destroy cars out of city's scope. */
void destroy_car(VEHICLE *vehicles, CITY *ryanville);

/* Sort cars so that further cars will be listed in lower indexes. */
void sort_car(VEHICLE *vehicles, CITY ryanville);

/* Swap vehicles. */
void swap_car(VEHICLE *vehicle1, VEHICLE *vehicle2);

/* Draw all vehicles given their current x, y coordinates and color */
void draw_traffic(VEHICLE *vehicles, CITY ryanville);

/* Draws a vehicle of car type. */
void draw_car(VEHICLE vehicle, CITY ryanville);

/* Draws a vehicle of sports type. */
void draw_sports(VEHICLE vehicle, CITY ryanville);

/* Draw a vehicle of truck type. */
void draw_truck(VEHICLE vehicle, CITY ryanville);

/* Display "Honk!" when the car is honking. */
void display_honk(int x, int y);

/* Generate an int between min and max (inclusive). */
int random(int min, int max);

/* Display error message if internal error is detected. */
void error(void);



/*********************************************************************

     MAIN

*********************************************************************/


int main(void)
{
	/* Event handling and buttons. */
	int  quit;              /* Exit application when this is TRUE. */
	int  event;             /* Most recent GP142 event. */
	char key_pressed;       /* Last keyboard character hit. */
	int  mouse_x, mouse_y;  /* Coordinates of latest mouse click. */

	/* City information. */
	CITY ryanville = {
		INIT_NUMCARS_ALLOWED,
		INIT_NUMCARS,
		INIT_TRAFFIC_MOVING,
		INIT_SPEED_LIMIT,
		INIT_PALETTE,
		INIT_HOURS,
		INIT_MINUTES };

	/* Array of all the cars in the city. */
	VEHICLE vehicles[MAX_VEHICLES + 1];


	/* Initialize random number generator. */
	srand(time(NULL));

	/* Initialize graphics package, clear the window, */
	/* turn logging off, and start animation. */
	GP142_open();
	GP142_logging(LOG_OFF);
	GP142_clear();
	GP142_animate(ANI_RUN);

	/* Draw initial landscape, buttons, and traffic. */
	draw_landscape(ryanville);
	draw_buttons(ryanville);
	draw_traffic(vehicles, ryanville);

	/* Main event loop: run until quit is TRUE. */
	quit = FALSE;

	while(!quit)
	{
		/* Get next event. */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_pressed);

		/* Perform appropriate action depending on kind of event. */
		switch (event) {
		case GP142_QUIT:
			/* Quit selected. */
			quit = TRUE;
			break;

		case GP142_KBD:
			/* Key pressed.  Not used. */
			break;

		case GP142_MOUSE:
			/* Mouse click.  Respond accordingly. */
			handle_button(mouse_x, mouse_y, &ryanville, vehicles);
			break;

		case GP142_PERIODIC:
			/* Timer interval event.  Update everything	*/
			/* that needs to change as time advances. */
			changetime(&ryanville);
			if(ryanville.traffic_moving)
				move_traffic(vehicles, &ryanville);
			break;
		}
		
		/* Redraw everything. */
		draw_landscape(ryanville);
		draw_buttons(ryanville);
		draw_traffic(vehicles, ryanville);
	}

	/* Terminate application. */
	GP142_close();

	return 0;
}


/*********************************************************************

    BACKGROUND LANDSCAPE

*********************************************************************/


/* Draw the background. */
void draw_landscape(CITY ryanville)
{
	int text_offset = -GP142_XMAX + 15; /* Horizontal offset of text. */
	int text_size   = 20;               /* Size of text font. */
	int sky_color;                      /* Color of the sky. */

	/* Draw the sky, color depends on the time. */
	if((ryanville.hour < HOUR_MORNING) ||
	   (ryanville.hour > HOUR_DUSK))
	{
		sky_color = NAVY_BLUE;
	}else{
		if((ryanville.hour == HOUR_MORNING) ||
		   (ryanville.hour == HOUR_DUSK))
		{
			sky_color = BLUE;
		}else{
			sky_color = ICE_BLUE;
		}
	}

	GP142_rectangleXY(sky_color, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, GP142_YMAX, FILL);

	if(sky_color == NAVY_BLUE)sprinkle_star();

	/* Draw the Sun or the Moon, depending on time. */
	draw_satellite(ryanville, sky_color);

	/* Draw some mountains. */
	GP142_triangleXY(MED_GRAY,
		-GP142_XMAX, HORIZON,
		GP142_XMAX, HORIZON,
		0, (4 * GP142_YMAX)/5 ,
		FILL);
	GP142_triangleXY(OLIVE,
		-( 5 * GP142_XMAX/4), HORIZON,
		(GP142_XMAX)/3  , HORIZON,
		-(3* GP142_XMAX)/4, (2 * GP142_YMAX)/3 ,
		FILL);
	GP142_triangleXY(FOREST_GREEN,
		-GP142_XMAX / 3 , HORIZON,
		GP142_XMAX, HORIZON,
		GP142_XMAX, (2 * GP142_YMAX)/3 ,
		FILL);
	
	/* Draw the ground and the roads. */
	GP142_rectangleXY(GREEN, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, HORIZON, FILL);
	GP142_rectangleXY(MED_GRAY,
		-GP142_XMAX, ROAD_TOP_Y + ROAD_WIDTH,
		GP142_XMAX, ROAD_TOP_Y - ROAD_WIDTH, FILL);
	GP142_rectangleXY(MED_GRAY,
		-GP142_XMAX, ROAD_BOTTOM_Y + ROAD_WIDTH,
		GP142_XMAX, ROAD_BOTTOM_Y - ROAD_WIDTH, FILL);
	draw_center_line(ROAD_TOP_Y);
	draw_center_line(ROAD_BOTTOM_Y);
}


/* Draw few randomly positioned stars. */
void sprinkle_star(void)
{
	int x, y;                /* Coordinates of the star. */
	int size;                /* Size of the star. */
	int i;                   /* Loop counter. */

	for(i = 1; i <= NUM_STARS; i++)
	{
		x = random(-GP142_XMAX, GP142_XMAX);
		y = random(HORIZON, GP142_YMAX);
		size = random(3, 10);

		GP142_printfXY(YELLOW, x, y, size, "*");
	}
}

/* Draw Sun/Moon.  Position calculated by time. */
void draw_satellite(CITY ryanville, int sky_color)
{
	int x, y;                /* Coordinates of the satellite. */
	double a;                /* Intermediate variable. */
	int i;                   /* Loop counter. */
	int ray_x, ray_y;        /* Ray vector from the Sun. */

	if((ryanville.hour >= HOUR_MORNING) &&
	   (ryanville.hour < HOUR_DUSK))
	{
		a = (double)(ryanville.hour) + (ryanville.minute / 60.0);
		x = (int)((12.0 - a) * GP142_XMAX / 6.0);
		a = (SATELLITE_RADIUS-GP142_YMAX)/(double)(pow(GP142_XMAX,2));
		y = (int)(a * x * x) + GP142_YMAX - SATELLITE_RADIUS;

		GP142_circleXY(YELLOW, x, y, SATELLITE_RADIUS);
		for(i = 1; i <= NUM_RAYS; i++)
		{
			ray_x = random(x-(3*SATELLITE_RADIUS),x+(3*SATELLITE_RADIUS));
			ray_y = random(y-(3*SATELLITE_RADIUS),y+(3*SATELLITE_RADIUS));

			GP142_lineXY(YELLOW, x, y, ray_x, ray_y, 2);
		}

		if(random(1,10) != 1)
		{
			GP142_circleXY(BLACK, x + (SATELLITE_RADIUS / 3),
				y + (SATELLITE_RADIUS / 3), 1);
			GP142_circleXY(BLACK, x - (SATELLITE_RADIUS / 3),
				y + (SATELLITE_RADIUS / 3), 1);
		}else{
			GP142_lineXY(BLACK,
				x + (SATELLITE_RADIUS / 3) - 1, y + (SATELLITE_RADIUS / 3),
				x + (SATELLITE_RADIUS / 3) + 1, y + (SATELLITE_RADIUS / 3),
				1);
			GP142_lineXY(BLACK,
				x - (SATELLITE_RADIUS / 3) - 1, y + (SATELLITE_RADIUS / 3),
				x - (SATELLITE_RADIUS / 3) + 1, y + (SATELLITE_RADIUS / 3),
				1);
		}

	}else{
		a = (double)(ryanville.hour) + (ryanville.minute / 60.0);
		if(ryanville.hour >= HOUR_DUSK)(a += -24.0);
		x = (int)(-a * GP142_XMAX / 6.0);
		a = (SATELLITE_RADIUS-GP142_YMAX)/(double)(pow(GP142_XMAX,2));
		y = (int)(a * x * x) + GP142_YMAX - SATELLITE_RADIUS;

		GP142_circleXY(WHITE, x, y, SATELLITE_RADIUS);
		GP142_circleXY(sky_color, x + (SATELLITE_RADIUS / 2),
			y + (SATELLITE_RADIUS / 2), (SATELLITE_RADIUS * 2) / 3);
	}
}


/* Draw dashed yellow road center line horizontally. */
void draw_center_line(int y)
{
	int x;                   /* Loop counter. */

	for(x = -GP142_XMAX; x <= GP142_XMAX; x++)
		if(((x + GP142_XMAX) % 20) < 10)
			GP142_pixelXY(YELLOW, x, y);
}



/*********************************************************************

    BUTTONS AND MOUSE CLICKS

*********************************************************************/


/* Draws the buttons. */
void draw_buttons(CITY ryanville)
{
	int i, temp;
	
	/* Draw rectangles for each button. */
	for(i = 0; i < NUM_BUTTONS; i++)
	{
		GP142_rectangleXY(BLACK,
			BUTTON_LEFT,
			BUTTON_UP-(BUTTON_SPACING+BUTTON_HEIGHT)*i,
			BUTTON_LEFT + BUTTON_WIDTH,
			BUTTON_UP-BUTTON_SPACING*i-BUTTON_HEIGHT*(i+1), 2);
	}
	
	/* Display button text. */
	i = BUTTON_UP-(BUTTON_HEIGHT/2)-(BUTTON_TEXT_SIZE/2);
	GP142_printfXY(FOREST_GREEN, BUTTON_TEXT_OFFSET, i,
		BUTTON_TEXT_SIZE, "GO");	
	
	i -= (BUTTON_HEIGHT + BUTTON_SPACING);
	GP142_printfXY(RED, BUTTON_TEXT_OFFSET, i,
		BUTTON_TEXT_SIZE, "STOP");
	
	i -= (BUTTON_HEIGHT + BUTTON_SPACING);
	GP142_printfXY(BLACK, BUTTON_TEXT_OFFSET, i,
		BUTTON_TEXT_SIZE, "REVERSE");

	/* Draw rectangles for each control button. */
	for(i = 0; i < NUM_CONTROL; i++)
	{
		GP142_rectangleXY(BLACK,
			CONTROL_LEFT,
			CONTROL_UP-(BUTTON_SPACING+CONTROL_HEIGHT)*i,
			CONTROL_LEFT + CONTROL_WIDTH,
			CONTROL_UP-BUTTON_SPACING*i-CONTROL_HEIGHT*(i+1), 2);
		GP142_rectangleXY(BLACK,
			CONTROL_BUTTON_LEFT,
			CONTROL_UP-(BUTTON_SPACING+CONTROL_HEIGHT)*i,
			CONTROL_BUTTON_LEFT + CONTROL_BUTTON_WIDTH,
			CONTROL_UP-BUTTON_SPACING*i-CONTROL_HEIGHT*(i+1), 2);
		GP142_lineXY(BLACK,
			CONTROL_BUTTON_LEFT,
			CONTROL_UP-BUTTON_SPACING*i-CONTROL_HEIGHT*i
			-CONTROL_HEIGHT/2,
			CONTROL_BUTTON_LEFT + CONTROL_BUTTON_WIDTH,
			CONTROL_UP-BUTTON_SPACING*i-CONTROL_HEIGHT*i
			-CONTROL_HEIGHT/2, 2);
		GP142_triangleXY(RED,
			2 + CONTROL_BUTTON_LEFT,
			2+CONTROL_UP-BUTTON_SPACING*i-CONTROL_HEIGHT*i
			-CONTROL_HEIGHT/2,
			-2 + CONTROL_BUTTON_LEFT + CONTROL_BUTTON_WIDTH,
			2+CONTROL_UP-BUTTON_SPACING*i-CONTROL_HEIGHT*i
			-CONTROL_HEIGHT/2,
			CONTROL_BUTTON_LEFT + CONTROL_BUTTON_WIDTH/2,
			-2+CONTROL_UP-BUTTON_SPACING*i-CONTROL_HEIGHT*i, FILL);
		GP142_triangleXY(RED,
			2 + CONTROL_BUTTON_LEFT,
			-2+CONTROL_UP-BUTTON_SPACING*i-CONTROL_HEIGHT*i-CONTROL_HEIGHT/2,
			-2 + CONTROL_BUTTON_LEFT + CONTROL_BUTTON_WIDTH,
			-2+CONTROL_UP-BUTTON_SPACING*i-CONTROL_HEIGHT*i-CONTROL_HEIGHT/2,
			CONTROL_BUTTON_LEFT + CONTROL_BUTTON_WIDTH/2,
			2 + CONTROL_UP - BUTTON_SPACING*i - CONTROL_HEIGHT*(i+1),
			FILL);
	}

	/* Display control button text. */
	i = CONTROL_UP-(CONTROL_HEIGHT/2)-(CONTROL_TEXT_SIZE/2);
	GP142_printfXY(BLACK, CONTROL_TEXT_OFFSET, i,
		CONTROL_TEXT_SIZE, "%d cars", ryanville.numcars_allowed);

	i -= (CONTROL_HEIGHT + BUTTON_SPACING);
	GP142_printfXY(BLACK, CONTROL_TEXT_OFFSET, i,
		CONTROL_TEXT_SIZE, "%d mph", ryanville.speed_limit);

	/* Display time. */
	GP142_rectangleXY(BLACK,
		TIME_LEFT,
		TIME_UP,
		TIME_LEFT + TIME_WIDTH,
		TIME_UP - TIME_HEIGHT, 2);

	i = TIME_UP - (TIME_HEIGHT/2) - (TIME_TEXT_SIZE/2);
	if(ryanville.hour == 12)
		temp = 12;
	else
		temp = ryanville.hour % 12;
	if(ryanville.minute >=10)
		GP142_printfXY(BLACK, TIME_TEXT_OFFSET, i,
			TIME_TEXT_SIZE, "%d:%d", temp, ryanville.minute);
	else
		GP142_printfXY(BLACK, TIME_TEXT_OFFSET, i,
			TIME_TEXT_SIZE, "%d:0%d", temp, ryanville.minute);

	/* Display palette. */
	GP142_rectangleXY(BLACK, PALETTE_LEFT, PALETTE_UP,
		PALETTE_LEFT + PALETTE_WIDTH, PALETTE_UP - PALETTE_HEIGHT, 2);
	for(i = 0; i < MAX_COLORS; i++)
	{
		GP142_rectangleXY(i,
			PALETTE_LEFT + (PALETTE_COLOR_WIDTH*i),
			PALETTE_UP + PALETTE_COLOR_WIDTH - PALETTE_HEIGHT,
			PALETTE_LEFT + (PALETTE_COLOR_WIDTH*(i+1)),
			PALETTE_UP - PALETTE_HEIGHT, FILL);
		GP142_rectangleXY(BLACK,
			PALETTE_LEFT + (PALETTE_COLOR_WIDTH*i),
			PALETTE_UP + PALETTE_COLOR_WIDTH - PALETTE_HEIGHT,
			PALETTE_LEFT + (PALETTE_COLOR_WIDTH*(i+1)),
			PALETTE_UP - PALETTE_HEIGHT, 1);
	}

	/* Display the current palette color chosen. */
	i = PALETTE_UP - (PALETTE_HEIGHT/2);
	switch(ryanville.palette)
	{
		case BLACK:
			GP142_printfXY(BLACK, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "BLACK");
			break;
		case WHITE:
			GP142_printfXY(WHITE, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "WHITE");
			break;
		case RED:
			GP142_printfXY(RED, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "RED");
			break;
		case GREEN:
			GP142_printfXY(BLACK, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "GREEN");
			break;
		case BLUE:
			GP142_printfXY(BLUE, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "BLUE");
			break;
		case YELLOW:
			GP142_printfXY(YELLOW, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "YELLOW");
			break;
		case MAGENTA:
			GP142_printfXY(MAGENTA, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "MAGENTA");
			break;
		case CYAN:
			GP142_printfXY(CYAN, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "CYAN");
			break;
		case PURPLE:
			GP142_printfXY(PURPLE, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "PURPLE");
			break;
		case NAVY_BLUE:
			GP142_printfXY(NAVY_BLUE, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "NAVY BLUE");
			break;
		case DUSTY_PLUM:
			GP142_printfXY(DUSTY_PLUM, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "DUSTY PLUM");
			break;
		case ICE_BLUE:
			GP142_printfXY(ICE_BLUE, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "ICE BLUE");
			break;
		case TURQUOISE:
			GP142_printfXY(TURQUOISE, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "TURQUOISE");
			break;
		case ORANGE:
			GP142_printfXY(ORANGE, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "ORANGE");
			break;
		case BROWN:
			GP142_printfXY(BROWN, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "BROWN");
			break;
		case PINK:
			GP142_printfXY(PINK, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "PINK");
			break;
		case CHALK:
			GP142_printfXY(CHALK, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "CHALK");
			break;
		case GOLD:
			GP142_printfXY(GOLD, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "GOLD");
			break;
		case PEACH:
			GP142_printfXY(PEACH, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "PEACH");
			break;
		case FOREST_GREEN:
			GP142_printfXY(FOREST_GREEN, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "FOREST GREEN");
			break;
		case SEA_GREEN:
			GP142_printfXY(SEA_GREEN, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "SEA GREEN");
			break;
		case OLIVE:
			GP142_printfXY(OLIVE, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "OLIVE");
			break;
		case MED_GRAY:
			GP142_printfXY(MED_GRAY, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "MEDIUM GRAY");
			break;
		case LT_GRAY:
			GP142_printfXY(LT_GRAY, PALETTE_TEXT_OFFSET, i,
				PALETTE_TEXT_SIZE, "LIGHT GRAY");
			break;
	}
}

/* Classify mouse click at (mouse_x,mouse_y) and return kind. */
int classify_mouse_click(int mouse_x, int mouse_y)
{
	/* Here are the button clicks. */
	if((mouse_x >= BUTTON_LEFT) &&
		(mouse_x <= BUTTON_LEFT + BUTTON_WIDTH))
	{
		if((mouse_y <= BUTTON_UP) &&
			(mouse_y >= BUTTON_UP - BUTTON_HEIGHT))
			return CLICK_GO;
		if((mouse_y <= BUTTON_UP - BUTTON_SPACING - BUTTON_HEIGHT) &&
			(mouse_y >= BUTTON_UP - BUTTON_SPACING - 2*BUTTON_HEIGHT))
			return CLICK_STOP;
		if((mouse_y <= BUTTON_UP - 2*BUTTON_SPACING - 2*BUTTON_HEIGHT) &&
			(mouse_y >= BUTTON_UP - 2*BUTTON_SPACING - 3*BUTTON_HEIGHT))
			return CLICK_REVERSE;
	}

	/* Here are the control button clicks. */
	if((mouse_x >= CONTROL_BUTTON_LEFT) &&
		(mouse_x <= CONTROL_BUTTON_LEFT + CONTROL_BUTTON_WIDTH))
	{
		if((mouse_y <= CONTROL_UP) &&
			(mouse_y >= CONTROL_UP - CONTROL_HEIGHT/2))
			return CLICK_NUMCAR_UP;
		if((mouse_y <= CONTROL_UP - CONTROL_HEIGHT/2) &&
			(mouse_y >= CONTROL_UP - CONTROL_HEIGHT))
			return CLICK_NUMCAR_DOWN;
		if((mouse_y <= CONTROL_UP - BUTTON_SPACING - CONTROL_HEIGHT) &&
			(mouse_y >= CONTROL_UP - BUTTON_SPACING - CONTROL_HEIGHT*3/2))
			return CLICK_SPEED_UP;
		if((mouse_y <= CONTROL_UP - BUTTON_SPACING - CONTROL_HEIGHT*3/2) &&
			(mouse_y >= CONTROL_UP - BUTTON_SPACING - CONTROL_HEIGHT*2))
			return CLICK_SPEED_DOWN;
	}

	/* Here are the palette clicks. */
	if((mouse_x >= PALETTE_LEFT) &&
		(mouse_x <= PALETTE_LEFT + PALETTE_COLOR_WIDTH*MAX_COLORS) &&
		(mouse_y <= -GP142_YMAX + BUTTON_SPACING + PALETTE_COLOR_WIDTH) &&
		(mouse_y >= -GP142_YMAX + BUTTON_SPACING))
		return CLICK_PALETTE;

	/* If it's not clicked anywere on the above buttons, the user */
	/* clicked on nothing. */
	return CLICK_NOWHERE;
}



/* Process button click at location x, y.  Update car speed */
/* direction, color, and whether traffic is moving as needed. */
void handle_button(int mouse_x, int mouse_y,
				   CITY *ryanville, VEHICLE *vehicles)
{
	int click_type;          /* What user clicked on. */
	int carindex;            /* Car user clicked on. */
	int i;                   /* Loop counter... need it to make */
	                         /* cars stop honking when stopped. */


	click_type = classify_mouse_click(mouse_x, mouse_y);
	
	switch(click_type)
	{
		case CLICK_NOWHERE:
			/* If user clicked on a car, change the car's color to */
			/* what ever is on the curren palette. */
			carindex = check_car(vehicles, *ryanville, mouse_x, mouse_y);
			if(carindex != -1)
				vehicles[carindex].color = ryanville->palette;
			break;

		case CLICK_GO:
			/* Make the traffic go. */
			ryanville->traffic_moving = FORWARD;
			break;

		case CLICK_STOP:
			/* Stop the traffic. */
			ryanville->traffic_moving = STOP;
			for(i = 0; i < ryanville->numcars; i++)
				vehicles[i].state = STATE_NORMAL;
			break;

		case CLICK_REVERSE:
			/* Everybody starts backing up. */
			ryanville->traffic_moving = REVERSE;
			break;

		case CLICK_NUMCAR_UP:
			/* Allow more cars in the city. */
			(ryanville->numcars_allowed)++;
			if(ryanville->numcars_allowed > MAX_VEHICLES)
				ryanville->numcars_allowed = MAX_VEHICLES;
			break;

		case CLICK_NUMCAR_DOWN:
			/* Allow less cars in the city. */
			(ryanville->numcars_allowed)--;
			if(ryanville->numcars_allowed < 0)
				ryanville->numcars_allowed = 0;
			break;

		case CLICK_SPEED_UP:
			/* Raise the speed of the cars. */
			(ryanville->speed_limit)++;
			if(ryanville->speed_limit > MAX_SPEED_LIMIT)
				ryanville->speed_limit = MAX_SPEED_LIMIT;
			break;

		case CLICK_SPEED_DOWN:
			/* Lower the speed of the cars. */
			(ryanville->speed_limit)--;
			if(ryanville->speed_limit < MIN_SPEED_LIMIT)
				ryanville->speed_limit = MIN_SPEED_LIMIT;
			break;

		case CLICK_PALETTE:
			/* Change the color on the palette. */
			ryanville->palette = color_clicked(mouse_x, mouse_y);
			break;
	}
}


/* Check to see if the coordinates x, y overlaps with another car. */
/* If it does overlaps, return the index of the car it overlaps */
/* with, but if it doesn't overlap, -1 is returned. */
int check_car(VEHICLE *vehicles, CITY ryanville, int mouse_x, int mouse_y)
{
	int i;

	for(i = 0; i < ryanville.numcars; i++)
	{
		switch(vehicles[i].type)
		{
			case CAR:
				if((mouse_x >= (vehicles[i].x - CAR_LENGTH/2)) &&
					(mouse_x <= (vehicles[i].x + CAR_LENGTH/2)) &&
					(mouse_y >= vehicles[i].y) &&
					(mouse_y <= (vehicles[i].y + CAR_HEIGHT)))
					return i;
				break;

			case SPORTS:
				if((mouse_x >= (vehicles[i].x - SPORTS_LENGTH/2)) &&
					(mouse_x <= (vehicles[i].x + SPORTS_LENGTH/2)) &&
					(mouse_y >= vehicles[i].y) &&
					(mouse_y <= (vehicles[i].y + SPORTS_HEIGHT)))
					return i;
				break;

			case TRUCK:
				if((mouse_x >= (vehicles[i].x - TRUCK_LENGTH/2)) &&
					(mouse_x <= (vehicles[i].x + TRUCK_LENGTH/2)) &&
					(mouse_y >= vehicles[i].y) &&
					(mouse_y <= (vehicles[i].y + TRUCK_HEIGHT)))
					return i;
				break;
		}
	}

	return -1;
}


/* Returns the color clicked in the palette. */
int color_clicked(int mouse_x, int mouse_y)
{
	return (mouse_x-BUTTON_SPACING+GP142_XMAX)/PALETTE_COLOR_WIDTH;
}



/*********************************************************************

    TRAFFIC AND OTHER ANIMATED OBJECTS

*********************************************************************/


/* Update the time. */
void changetime(CITY *ryanville)
{
	ryanville->minute += MINUTE_INCREMENT;

	if(ryanville->minute >= 60)
	{
		ryanville->minute += -60;
		(ryanville->hour)++;

		if(ryanville->hour == 24)
			ryanville->hour = 0;
	}
}


/* Update all vehicle x, y coordinates, given current x, y. */
void move_traffic(VEHICLE *vehicles, CITY *ryanville)
{
	int i;                  /* Loop counter. */
	int test_x;             /* Location car is about to occupy. */
	int test_result;        /* Was the space empty? */

	/* If more car is allowed, let a car in the city. */
	if(ryanville->numcars < ryanville->numcars_allowed)
		create_car(vehicles, ryanville);

	for(i = 0; i < ryanville->numcars; i++)
	{
		if(ryanville->traffic_moving == FORWARD)
		{
			if(vehicles[i].direction == 'e')
			{
				test_x = vehicles[i].x + ryanville->speed_limit/5;

				switch(vehicles[i].type)
				{
					case CAR:
						break;

					case SPORTS:
						test_x += 2;
						break;

					case TRUCK:
						test_x -= 1;
						break;
				}

				test_result = check_car(vehicles, *ryanville,
					test_x + TRUCK_LENGTH/2,
					vehicles[i].y + CAR_HEIGHT/2);
				if(test_result == -1)
				{
					vehicles[i].x = test_x;
				}else{
					vehicles[i].state = STATE_HONKING;
					vehicles[i].state_count = 3;
				}
			}

			if(vehicles[i].direction == 'w')
			{
				test_x = vehicles[i].x - ryanville->speed_limit/5;

				switch(vehicles[i].type)
				{
					case CAR:
						break;

					case SPORTS:
						test_x -= 2;
						break;

					case TRUCK:
						test_x += 1;
						break;
				}

				test_result = check_car(vehicles, *ryanville,
					test_x - TRUCK_LENGTH/2,
					vehicles[i].y + CAR_HEIGHT/2);
				if(test_result == -1)
				{
					vehicles[i].x = test_x;
				}else{
					vehicles[i].state = STATE_HONKING;
					vehicles[i].state_count = 3;
				}
			}
		}

		if(ryanville->traffic_moving == REVERSE)
		{
			if(vehicles[i].direction == 'e')
			{
				vehicles[i].x -= 2;
			}

			if(vehicles[i].direction == 'w')
			{
				vehicles[i].x += 2;
			}
		}

		if(vehicles[i].state == STATE_HONKING)
		{
			(vehicles[i].state_count)--;
			if(vehicles[i].state_count <= 0)
				vehicles[i].state = STATE_NORMAL;
		}
	}

	/* Destroy cars that went out of the city's scope. */
	destroy_car(vehicles, ryanville);

	return;
}


/* Randomly generate a car. */
void create_car(VEHICLE *vehicles, CITY *ryanville)
{
	int i;
	int spot_open = TRUE;
	int path_to_check;
	int type_of_car;

	/* Check a path to drive in from, and if there's room, */
	/* create a car. */
	path_to_check = random(1,4);
	for(i = 0; i < ryanville->numcars; i++)
	{
		if((path_to_check == 1) &&
			(vehicles[i].y == ROAD_TOP_Y - ROAD_WIDTH/2) &&
			(vehicles[i].x < -GP142_XMAX+TRUCK_LENGTH))
			spot_open = FALSE;
		if((path_to_check == 2) &&
			(vehicles[i].y == ROAD_BOTTOM_Y - ROAD_WIDTH/2) &&
			(vehicles[i].x < -GP142_XMAX+TRUCK_LENGTH))
			spot_open = FALSE;
		if((path_to_check == 3) &&
			(vehicles[i].y == ROAD_TOP_Y + ROAD_WIDTH/2) &&
			(vehicles[i].x > GP142_XMAX-TRUCK_LENGTH))
			spot_open = FALSE;
		if((path_to_check == 4) &&
			(vehicles[i].y == ROAD_BOTTOM_Y + ROAD_WIDTH/2) &&
			(vehicles[i].x > GP142_XMAX-TRUCK_LENGTH))
			spot_open = FALSE;
	}

	if(!spot_open)
		return;

	switch(path_to_check)
	{
		case 1:
			vehicles[i].x = -GP142_XMAX-TRUCK_LENGTH/2;
			vehicles[i].y = ROAD_TOP_Y-ROAD_WIDTH/2;
			vehicles[i].direction = 'e';
			break;

		case 2:
			vehicles[i].x = -GP142_XMAX-TRUCK_LENGTH/2;
			vehicles[i].y = ROAD_BOTTOM_Y-ROAD_WIDTH/2;
			vehicles[i].direction = 'e';
			break;

		case 3:
			vehicles[i].x = GP142_XMAX+TRUCK_LENGTH/2;
			vehicles[i].y = ROAD_TOP_Y+ROAD_WIDTH/2;
			vehicles[i].direction = 'w';
			break;

		case 4:
			vehicles[i].x = GP142_XMAX+TRUCK_LENGTH/2;
			vehicles[i].y = ROAD_BOTTOM_Y+ROAD_WIDTH/2;
			vehicles[i].direction = 'w';
			break;
	}

	/* Randomly decide on type of the car, depending on their */
	/* occurance chance. */
	type_of_car = random(1,100);
	if(type_of_car <= CHANCE_CAR)
		vehicles[i].type = CAR;
	else if(type_of_car <= (CHANCE_CAR + CHANCE_SPORTS))
		vehicles[i].type = SPORTS;
	else
		vehicles[i].type = TRUCK;

	/* Randomly decide on their color. */
	vehicles[i].color = random(0, MAX_COLORS - 1);

	vehicles[i].state = STATE_NORMAL;
	vehicles[i].state_count = 0;

	/* Now we have an extra car in the city. */
	(ryanville->numcars)++;

	return;
}


/* Destroy cars out of scope. */
void destroy_car(VEHICLE *vehicles, CITY *ryanville)
{
	int i, j;

	for(i = 0; i < ryanville->numcars; i++)
	{
		if((vehicles[i].x < -GP142_XMAX-TRUCK_LENGTH) ||
			(vehicles[i].x > GP142_XMAX+TRUCK_LENGTH))
		{
			for(j = i; j < ryanville->numcars; j++)
				swap_car(&vehicles[j], &vehicles[j+1]);
			(ryanville->numcars)--;
			i--;
		}
	}
}


/* Sort cars so that further cars will be listed in lower indexes. */
void sort_car(VEHICLE *vehicles, CITY ryanville)
{
	int sorted = FALSE;
	int i, j;
	int furthest;

	for(i = 0; i < ryanville.numcars; i++)
	{
		furthest = i;
		for(j = i + 1; j < ryanville.numcars; j++)
		{
			if(vehicles[j].y > vehicles[furthest].y)
				furthest = j;
		}
		swap_car(&vehicles[i], &vehicles[furthest]);
	}
}


/* Swap vehicles. */
void swap_car(VEHICLE *vehicle1, VEHICLE *vehicle2)
{
	VEHICLE temp;

	temp = *vehicle1;
	*vehicle1 = *vehicle2;
	*vehicle2 = temp;
}


/* Draw all vehicles. */
void draw_traffic(VEHICLE *vehicles, CITY ryanville)
{
	int i;

	sort_car(vehicles, ryanville);

	for(i = 0; i < ryanville.numcars; i++)
	{
		if(vehicles[i].type == CAR)
			draw_car(vehicles[i], ryanville);
		if(vehicles[i].type == SPORTS)
			draw_sports(vehicles[i], ryanville);
		if(vehicles[i].type == TRUCK)
			draw_truck(vehicles[i], ryanville);
	}

	return;
}


/* Draws a single car. */
void draw_car(VEHICLE vehicle, CITY ryanville)
{
	/* Turn the head light on if it's dark. */
	if((ryanville.hour >= HOUR_DUSK) ||
		(ryanville.hour <= HOUR_MORNING))
	{
		if(vehicle.direction == 'w')
			GP142_ovalXY(YELLOW, vehicle.x - CAR_LENGTH/2 - 8,
				vehicle.y + 5, vehicle.x - CAR_LENGTH/2 + 8,
				vehicle.y, FILL);
		if(vehicle.direction == 'e')
			GP142_ovalXY(YELLOW, vehicle.x + CAR_LENGTH/2 - 8,
				vehicle.y + 5, vehicle.x + CAR_LENGTH/2 + 8,
				vehicle.y, FILL);
	}

	GP142_rectangleXY(vehicle.color, vehicle.x - CAR_LENGTH/2,
		vehicle.y + CAR_HEIGHT/4, vehicle.x + CAR_LENGTH/2,
		vehicle.y + CAR_HEIGHT*2/3, FILL);
	GP142_rectangleXY(vehicle.color, vehicle.x - CAR_LENGTH/4,
		vehicle.y + CAR_HEIGHT*2/3, vehicle.x + CAR_LENGTH/4,
		vehicle.y + CAR_HEIGHT, FILL);
	GP142_circleXY(BLACK, vehicle.x - CAR_LENGTH/4,
		vehicle.y + CAR_HEIGHT/4, CAR_HEIGHT/4);
	GP142_circleXY(BLACK, vehicle.x + CAR_LENGTH/4,
		vehicle.y + CAR_HEIGHT/4, CAR_HEIGHT/4);

	/* Turn the brake lights on if not moving forward. */
	if(ryanville.traffic_moving != FORWARD)
	{
		if(vehicle.direction == 'e')
			GP142_circleXY(RED, vehicle.x - CAR_LENGTH/2,
				vehicle.y + CAR_HEIGHT/2, 2);
		if(vehicle.direction == 'w')
			GP142_circleXY(RED, vehicle.x + CAR_LENGTH/2,
				vehicle.y + CAR_HEIGHT/2, 2);
	}

	/* If the car is honking, display the "Honk!" */
	if(vehicle.state == STATE_HONKING)
		display_honk(vehicle.x, vehicle.y);

	return;
}


/* Draws a vehicle of sports type. */
void draw_sports(VEHICLE vehicle, CITY ryanville)
{
	/* Turn the head light on if it's dark. */
	if((ryanville.hour >= HOUR_DUSK) ||
		(ryanville.hour <= HOUR_MORNING))
	{
		if(vehicle.direction == 'w')
			GP142_ovalXY(YELLOW, vehicle.x - SPORTS_LENGTH/2 - 8,
				vehicle.y + 5, vehicle.x - SPORTS_LENGTH/2 + 8,
				vehicle.y, FILL);
		if(vehicle.direction == 'e')
			GP142_ovalXY(YELLOW, vehicle.x + SPORTS_LENGTH/2 - 8,
				vehicle.y + 5, vehicle.x + SPORTS_LENGTH/2 + 8,
				vehicle.y, FILL);
	}

	if(vehicle.direction == 'e')
	{
		GP142_triangleXY(vehicle.color, vehicle.x - SPORTS_LENGTH/2,
			vehicle.y + SPORTS_HEIGHT/4, vehicle.x - SPORTS_LENGTH/2,
			vehicle.y + SPORTS_HEIGHT, vehicle.x + SPORTS_LENGTH/2,
			vehicle.y + SPORTS_HEIGHT/4, FILL);
		GP142_ovalXY(CYAN, vehicle.x - SPORTS_LENGTH/4,
			vehicle.y + SPORTS_HEIGHT*5/6, vehicle.x + SPORTS_LENGTH/4,
			vehicle.y + SPORTS_HEIGHT*1/2, FILL);
		GP142_circleXY(BLACK, vehicle.x - SPORTS_LENGTH/4,
			vehicle.y + SPORTS_HEIGHT/4, SPORTS_HEIGHT/4);
		GP142_circleXY(BLACK, vehicle.x + SPORTS_LENGTH/4,
			vehicle.y + SPORTS_HEIGHT/4, SPORTS_HEIGHT/4);
	}

	if(vehicle.direction == 'w')
	{
		GP142_triangleXY(vehicle.color, vehicle.x - SPORTS_LENGTH/2,
			vehicle.y + SPORTS_HEIGHT/4, vehicle.x + SPORTS_LENGTH/2,
			vehicle.y + SPORTS_HEIGHT, vehicle.x + SPORTS_LENGTH/2,
			vehicle.y + SPORTS_HEIGHT/4, FILL);
		GP142_ovalXY(CYAN, vehicle.x - SPORTS_LENGTH/4,
			vehicle.y + SPORTS_HEIGHT*5/6, vehicle.x + SPORTS_LENGTH/4,
			vehicle.y + SPORTS_HEIGHT*1/2, FILL);
		GP142_circleXY(BLACK, vehicle.x - SPORTS_LENGTH/4,
			vehicle.y + SPORTS_HEIGHT/4, SPORTS_HEIGHT/4);
		GP142_circleXY(BLACK, vehicle.x + SPORTS_LENGTH/4,
			vehicle.y + SPORTS_HEIGHT/4, SPORTS_HEIGHT/4);
	}

	/* Turn the brake lights on if not moving forward. */
	if(ryanville.traffic_moving != FORWARD)
	{
		if(vehicle.direction == 'e')
			GP142_circleXY(RED, vehicle.x - SPORTS_LENGTH/2,
				vehicle.y + SPORTS_HEIGHT/2, 2);
		if(vehicle.direction == 'w')
			GP142_circleXY(RED, vehicle.x + SPORTS_LENGTH/2,
				vehicle.y + SPORTS_HEIGHT/2, 2);
	}

	/* If the car is honking, display the "Honk!" */
	if(vehicle.state == STATE_HONKING)
		display_honk(vehicle.x, vehicle.y);

	return;
}


/* Draw a vehicle of truck type. */
void draw_truck(VEHICLE vehicle, CITY ryanville)
{
	/* Turn the head light on if it's dark. */
	if((ryanville.hour >= HOUR_DUSK) ||
		(ryanville.hour <= HOUR_MORNING))
	{
		if(vehicle.direction == 'w')
			GP142_ovalXY(YELLOW, vehicle.x - TRUCK_LENGTH/2 - 8,
				vehicle.y + 5, vehicle.x - TRUCK_LENGTH/2 + 8,
				vehicle.y, FILL);
		if(vehicle.direction == 'e')
			GP142_ovalXY(YELLOW, vehicle.x + TRUCK_LENGTH/2 - 8,
				vehicle.y + 5, vehicle.x + TRUCK_LENGTH/2 + 8,
				vehicle.y, FILL);
	}

	if(vehicle.direction == 'e')
	{
		GP142_rectangleXY(LT_GRAY, vehicle.x - TRUCK_LENGTH/2,
			vehicle.y + TRUCK_HEIGHT, vehicle.x + TRUCK_LENGTH/4,
			vehicle.y + TRUCK_HEIGHT/4, FILL);
		GP142_rectangleXY(vehicle.color, vehicle.x + TRUCK_LENGTH/4,
			vehicle.y + TRUCK_HEIGHT*3/4, vehicle.x + TRUCK_LENGTH/2,
			vehicle.y + TRUCK_HEIGHT/4, FILL);
		GP142_circleXY(BLACK, vehicle.x - TRUCK_LENGTH/4,
			vehicle.y + TRUCK_HEIGHT/4, TRUCK_HEIGHT/4);
		GP142_circleXY(BLACK, vehicle.x + TRUCK_LENGTH/4,
			vehicle.y + TRUCK_HEIGHT/4, TRUCK_HEIGHT/4);
	}

	if(vehicle.direction == 'w')
	{
		GP142_rectangleXY(LT_GRAY, vehicle.x + TRUCK_LENGTH/2,
			vehicle.y + TRUCK_HEIGHT, vehicle.x - TRUCK_LENGTH/4,
			vehicle.y + TRUCK_HEIGHT/4, FILL);
		GP142_rectangleXY(vehicle.color, vehicle.x - TRUCK_LENGTH/4,
			vehicle.y + TRUCK_HEIGHT*3/4, vehicle.x - TRUCK_LENGTH/2,
			vehicle.y + TRUCK_HEIGHT/4, FILL);
		GP142_circleXY(BLACK, vehicle.x - TRUCK_LENGTH/4,
			vehicle.y + TRUCK_HEIGHT/4, TRUCK_HEIGHT/4);
		GP142_circleXY(BLACK, vehicle.x + TRUCK_LENGTH/4,
			vehicle.y + TRUCK_HEIGHT/4, TRUCK_HEIGHT/4);
	}

	/* Turn the brake lights on if not moving forward. */
	if(ryanville.traffic_moving != FORWARD)
	{
		if(vehicle.direction == 'e')
			GP142_circleXY(RED, vehicle.x - TRUCK_LENGTH/2,
				vehicle.y + TRUCK_HEIGHT/2, 2);
		if(vehicle.direction == 'w')
			GP142_circleXY(RED, vehicle.x + TRUCK_LENGTH/2,
				vehicle.y + TRUCK_HEIGHT/2, 2);
	}

	/* If the car is honking, display the "Honk!" */
	if(vehicle.state == STATE_HONKING)
		display_honk(vehicle.x, vehicle.y);

	return;
}

/* Display "Honk!" when the car is honking. */
void display_honk(int x, int y)
{
	GP142_ovalXY(WHITE, x - 15, y + 45, x + 15, y + 30, FILL);
	GP142_printfXY(BLACK, x - 12, y + 30, 10, "Honk!");
}



/*********************************************************************

    OTHER

*********************************************************************/


/* Generate an int between min and max (inclusive). */
int random(int min, int max)
{
	return (min + (rand() % (max + 1 - min)));
}


/* Display error message if internal error is detected. */
void error(void)
{
	GP142_printfXY(RED, 0, 0, 16, "ERROR!");
}